#ifndef _ComSyncDeviceInfo_h
#define _ComSyncDeviceInfo_h


typedef enum {
	NONE_Parity,
	ODD_Parity,
	EVEN_Parity = 3
}_Parity;


typedef enum {
	CRC_NONE,
	CRC_16,
	CRC_POLY
}_CRC;

typedef enum {	
	V11,
	EIA530A,
	EIA530,
	X21,
	V35,
	RS449,
	RS232
}_LIM;

// Values for 'rx_bits' field
typedef enum {				
	RX_5,
	RX_7,
	RX_6,	
	RX_8
}_RX_BITS;

// Values for 'sync_mode' field
typedef enum {				
	SYNC_8_MODE,
	SYNC_16_MODE,
	SDLC_MODE,
	EXT_SYNC_MODE
}_SYNC_MODE;

// Values for 'tx_bits' field
typedef enum {				
	TX_5,
	TX_7,
	TX_6,
	TX_8
}_TX_BITS;

// Values for 'data_encoding' field
typedef enum {				
	NoEncoding,
	NRZ,
	NRZI,
	FM1,
	FM0,
	MANCHESTER
}_DATA_ENCODING;

// Values for 'trxc_source' field
typedef enum {				
	TRXC_XTAL,
	TRXC_TXCLK,
	TRXC_BRG,
	TRXC_DPLL
}_TRXC_SRC;
 
// Values for 'tx_clock' field
typedef enum {				
	TXCLK_RTXC,
	TXCLK_TRXC,
	TXCLK_BRG,
	TXCLK_DPLL
}_TX_CLK;

// Values for 'rx_clock' field
typedef enum {				
	RXCLK_RTXC,
	RXCLK_TRXC,
	RXCLK_BRG,
	RXCLK_DPLL
}_RX_CLK;

typedef enum {				
	None,
	Single,
	Multiple	
}SearchMode;

#pragma pack(1)

typedef union {
    struct {        
        UCHAR SyncChar1;
        UCHAR SyncChar2;        
    } w;
    USHORT SyncChar;
} _SyncCharacter;


#define MAX_CHARS_EVENT 20

#define ACTION_RX_HUNT				0x00000001
#define ACTION_RX_DISCARD			0x00000002
#define ACTION_RX_EXCLUDE_CRC		0x00000004
#define ACTION_RX_RESUME_TX		0x00000008
#define ACTION_RX_EVENT				0x00000010
#define ACTION_RX_SET_RTS			0x00000020
#define ACTION_RX_CLEAR_RTS		0x00000040
#define ACTION_RX_SET_DTR			0x00000080
#define ACTION_RX_CLEAR_DTR		0x00000100
#define ACTION_RX_READ_RETURN		0x00000200
#define ACTION_RX_CHECK_CRC		0x00000400

typedef struct {
	UCHAR	ch; // control character to be monitored for specific action	
	ULONG	Action;	// defined above by 	ACTION_RX_***, action can be ored to take multiple actions
}_RxEvent;


typedef struct {
	_RxEvent	RxEvent[MAX_CHARS_EVENT];	
	ULONG NumberValidChars; // number of characters from RxEvent array to be monitored
}_RxCharEvent,*PRxCharEvent;

typedef struct {
	UCHAR ch[MAX_CHARS_EVENT]; // control character to be excluded from CRC calculation
	ULONG NumberValidChars; // number of characters from ch array to be excluded from CRC calculation
}_TxCrcExclude, *PTxCrcExclude;


typedef struct {
	PUCHAR Buf1; // buffer pointer to be allocated
	PUCHAR Buf2; // buffer pointer to be allocated
	PUCHAR Buf3; // buffer pointer to be allocated
	PUCHAR Buf4; // buffer pointer to be allocated
	PUCHAR Buf5; // buffer pointer to be allocated
	PUCHAR Buf6; // buffer pointer to be allocated
	PUCHAR Buf7; // buffer pointer to be allocated
	PUCHAR Buf8; // buffer pointer to be allocated
	UCHAR  Var1; // stoarge variable
	UCHAR  Var2; // stoarge variable
	UCHAR  Var3; // stoarge variable
	UCHAR  Var4; // stoarge variable
	UCHAR  Var5; // stoarge variable
	UCHAR  Var6; // stoarge variable
	UCHAR  Var7; // stoarge variable
	UCHAR  Var8; // stoarge variable
}_Protocol, *pProtocol;

typedef struct _ComSyncDeviceInfo {    
    ULONG                   ConfigVersion;     /* Version of this structure */

    _LIM     Lim;        /* Line Interface Mode        */
    
    _SYNC_MODE					 Framing;            /* Mono/BiSync, SDLC or Raw     */
	 
    _RX_BITS					 RxCharacterSize;    /* Character Size (5-8)         */    
	 _TX_BITS					 TxCharacterSize;    /* Character Size (5-8)         */    
    _Parity						 Parity;				   /* parity information			*/
	 
	 BOOLEAN						 AutoDcdCts;		   /* flag to enable DCD&CTS to become
																/* Rx&Tx enabler respectively*/    	  
	 /* Bit Rate Generator */
    /* Rx & Tx Clocks */
	 ULONG                   Oscillator;   /* Oscillator frequency     */
    _RX_CLK						 RxClock;      /* Receive Clock Source        */
    _TX_CLK						 TxClock;      /* Transmit Clock Source       */
	 _TRXC_SRC					 TRxCSource;	/* Values for 'trxc_source' field */
    BOOLEAN                 TRxCIO;			/* if TxClock is output, else input*/    
    ULONG                   BitRate;      /* Bit Rate						     */            
    _DATA_ENCODING			 ClockEncoding; /* Clock Encoding Method  */	 

    /* CRC */
    _CRC							 Crc;					/* CRC polynomial */
    BOOLEAN                 CrcPresetIO;   /* initial condition CRC checker    */

    BOOLEAN						 IdleSync;     /* flag to indicate sync chars as idle chars*/        
    
    SearchMode				    SdlcAddressSearchMode;	/* flag to enable SDLC address searching   */
    UCHAR                   SdlcAddress;           /* Receive Address [SDLC]          */
	     
	 BOOLEAN						 AutoRTSDeactivation; /* RTS will be deactivated at the end of frame */    
    
    BOOLEAN						 SyncCharSize6Or12 ; /* Sync Character Size (6, 12 bits)  */
	 _SyncCharacter          SyncCharacter;		/* sync charaters for mono & bisync */
	 
	 BOOLEAN						 InhibitSyncLoad;	/* this will inhibit loading sync character */
	 	 
	 _RxCharEvent RxCharEventArray; /* Action of received character for protocol implementation */

	 _TxCrcExclude	TxCrcExclude; /* CRC exclusion on specific character on Tx */

	 BOOLEAN		  				 MergeFrames;			/* if writes should be merged	*/	    
            
	 /* flag whether application wants DMA transfer or not */
	 BOOLEAN UseDmaRead; // DMA on receive
	 BOOLEAN UseDmaWrite; // DMA on transmit	

	 /* Diagnostic facility	*/
	 BOOLEAN                 Loopback;          /* if in local loopback mode    */

	 _Protocol				 Protocol;

	 UCHAR Reserved[100]; /* Reserved for future use */

} ComSyncDeviceInfo, *pComSyncDeviceInfo;



typedef struct {
	UCHAR Index; // 0 based index of ESCC write registers
	UCHAR Value; // value to be written to that regsiter
	BOOLEAN Prime; // flag to indicate whether register 7 prime is being written, index must be 7
}Register, *PRegister;


/*
	ReadTotalTimeoutMultiplier 
		Specifies the multiplier, in milliseconds, used to calculate the total time-out period 
		for read operations. For each read operation, this value is multiplied by the requested 
		number of bytes to be read. 
	
  ReadTotalTimeoutConstant 
		Specifies the constant, in milliseconds, used to calculate the total time-out period for 
		read operations. For each read operation, this value is added to the product of the 
		ReadTotalTimeoutMultiplier member and the requested number of bytes. 

	A value of zero for both the ReadTotalTimeoutMultiplier  
	member indicates that total time-outs are not used for read operations. 
*/
typedef struct _COMSYNC_TIMEOUTS {    
    ULONG ReadTotalTimeoutMultiplier;
    ULONG ReadTotalTimeoutConstant;    
    } COMSYNC_TIMEOUTS,*PCOMSYNC_TIMEOUTS;


#pragma pack()

#define FILE_DEVICE_CTI  0x00008000


/*
	This IOCTL will configure Comsync driver

	- Input buffer is ComSyncDeviceInfo structure.	   
	- Output buffer is none.	   
*/
#define IOCTL_COMSYNC_SET_CONFIGURATION \
	CTL_CODE(FILE_DEVICE_CTI, 0x800, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This IOCTL will return the current configuration of Comsync driver
	
	- Input buffer is None.	   
	- Output buffer is ComSyncDeviceInfo structure.	   .	   
*/
#define IOCTL_COMSYNC_GET_CONFIGURATION \
	CTL_CODE(FILE_DEVICE_CTI, 0x801, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This IOCTL will return the value of read registers of ESCC, the main
	communication controller of Comsync hardware. See zilog ESCC manual
	for description of read regsiters.

	- Input buffer is port index of 85230, buffer size UCHAR.	   
	- Output buffer is port's value, buffer size UCHAR.
*/
#define IOCTL_COMSYNC_READ_PORT \
    CTL_CODE(FILE_DEVICE_CTI, 0x802, METHOD_BUFFERED, FILE_READ_ACCESS)

/*
	This IOCTL will write values to ESCC write registers,the main
	communication controller of Comsync hardware. See zilog ESCC manual
	for description of read regsiters.

	- Input buffer is Register structure.	    
	- Output buffer is none.
*/
#define IOCTL_COMSYNC_WRITE_PORT \
    CTL_CODE(FILE_DEVICE_CTI, 0x803, METHOD_BUFFERED, FILE_WRITE_ACCESS)


#define EVENT_FRAME_CRC_ERROR 	0x00000001 // event if CRC flag is set
#define EVENT_OVERRUN_ERROR 		0x00000002 // event if overrun error occurs
#define EVENT_PARITY_ERROR	 		0x00000004 // event if parity error occurs
#define EVENT_FRAME_END_SDLC 		0x00000008 // event if SDLC frame end occurs

#define EVENT_ZERO_COUNT			0x00000010 // event if zero count interrupt occurs
#define EVENT_DCD		 				0x00000020 // event if DCD changes state
#define EVENT_CTS 					0x00000040 // event if CTS changes state
#define EVENT_SYNC_HUNT 			0x00000080 // event if hunt interrupt occurs
#define EVENT_BREAK_ABORT			0x00000100 // event if break interrupt occurs
#define EVENT_TX_UNDERRUN			0x00000200 // event if Tx Underrun interrupt occurs
/*
	This IOCTL will return when any event defined EVENT_*** will happen.

	- Input buffer is wait mask defined by EVENT_***, buffer size ULONG 					 
	- Output buffer is mask occured, buffer size ULONG.
	- mask value can be ored
*/
#define IOCTL_COMSYNC_WAIT_ON_EVENTS \
	CTL_CODE(FILE_DEVICE_CTI, 0x804, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not
	return until a charater is received.
*/
#define IOCTL_COMSYNC_RX_CHARS \
	CTL_CODE(FILE_DEVICE_CTI, 0x805, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will 
	force ESCC into hunt mode.
*/
#define IOCTL_COMSYNC_FORCE_HUNT \
	CTL_CODE(FILE_DEVICE_CTI, 0x806, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will hold transmission.	
*/
#define IOCTL_COMSYNC_TX_HOLD \
	CTL_CODE(FILE_DEVICE_CTI, 0x807, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will start transmission
	if it is on hold currently.	
*/
#define IOCTL_COMSYNC_TX_RESUME \
	CTL_CODE(FILE_DEVICE_CTI, 0x808, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not
	return until a frame error occured.
*/
#define IOCTL_COMSYNC_EVENT_FRAME_CRC_ERROR \
	CTL_CODE(FILE_DEVICE_CTI, 0x809, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not
	return until a overrun error occured.
*/
#define IOCTL_COMSYNC_EVENT_OVERRUN_ERROR \
	CTL_CODE(FILE_DEVICE_CTI, 0x80a, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not
	return until a parity error occured.
*/
#define IOCTL_COMSYNC_EVENT_PARITY_ERROR \
	CTL_CODE(FILE_DEVICE_CTI, 0x80b, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not 
	return until a frame end SDLC event occured.
*/
#define IOCTL_COMSYNC_EVENT_FRAME_END_SDLC \
	CTL_CODE(FILE_DEVICE_CTI, 0x80c, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not 
	return until a zero count occured.
*/
#define IOCTL_COMSYNC_EVENT_ZERO_COUNT \
	CTL_CODE(FILE_DEVICE_CTI, 0x80d, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not 
	return until DCD occured.
*/
#define IOCTL_COMSYNC_EVENT_DCD \
	CTL_CODE(FILE_DEVICE_CTI, 0x80e, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not 
	return until CTS occured.
*/
#define IOCTL_COMSYNC_EVENT_CTS \
	CTL_CODE(FILE_DEVICE_CTI, 0x80f, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not 
	return until hunt occured, see ESCC manual on hunt interrupt.
*/
#define IOCTL_COMSYNC_EVENT_SYNC_HUNT \
	CTL_CODE(FILE_DEVICE_CTI, 0x810, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not 
	return until break or abort occured, see ESCC manual on break or abort interrupt.
*/

#define IOCTL_COMSYNC_EVENT_BREAK_ABORT \
	CTL_CODE(FILE_DEVICE_CTI, 0x811, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will not 
	return until Tx underrun occured, see ESCC manual on underrun interrupt.
*/

#define IOCTL_COMSYNC_EVENT_TX_UNDERRUN \
	CTL_CODE(FILE_DEVICE_CTI, 0x812, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will call GenericRoutine1.
	This is available by kernel plugin code.
*/
#define IOCTL_COMSYNC_GENERIC1 \
	CTL_CODE(FILE_DEVICE_CTI, 0x813, METHOD_BUFFERED, FILE_ANY_ACCESS)

/* 
	There is no input\ouput buffer. This is a command IOCTL that will call GenericRoutine2.
	This is available by kernel plugin code.
*/
#define IOCTL_COMSYNC_GENERIC2\
	CTL_CODE(FILE_DEVICE_CTI, 0x814, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This will set timeout for read. 

	- Input buffer is COMSYNC_TIMEOUTS.	    
	- Output buffer is none.	
*/
#define IOCTL_COMSYNC_SET_TIMEOUTS\
	CTL_CODE(FILE_DEVICE_CTI, 0x815, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This will return timeout for read. 

	- Input buffer is none.	    
	- Output buffer is COMSYNC_TIMEOUTS.	
*/
#define IOCTL_COMSYNC_GET_TIMEOUTS\
	CTL_CODE(FILE_DEVICE_CTI, 0x816, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This will work if RxCharEventArray is defined for ACTION_RX_EVENT. This IOCTL will return 
	when the character in RxCharEventArray matches with a received character.
	- Input buffer is none.	    
	- Output buffer is none.	
*/
#define IOCTL_COMSYNC_RX_CHAR_SPECIAL\
	CTL_CODE(FILE_DEVICE_CTI, 0x817, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This IOCTL will raise DTR. It can not be used if DMA is effective.

	- Input buffer is none.	    
	- Output buffer is none.	
	
*/
#define IOCTL_COMSYNC_SET_DTR\
	CTL_CODE(FILE_DEVICE_CTI, 0x818, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This IOCTL will lower DTR. It can not be used if DMA is effective.

	- Input buffer is none.	    
	- Output buffer is none.	
	
*/
#define IOCTL_COMSYNC_CLEAR_DTR\
	CTL_CODE(FILE_DEVICE_CTI, 0x819, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This IOCTL will raise RTS.

	- Input buffer is none.	    
	- Output buffer is none.	
*/
#define IOCTL_COMSYNC_SET_RTS\
	CTL_CODE(FILE_DEVICE_CTI, 0x81a, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This IOCTL will lower RTS.

	- Input buffer is none.	    
	- Output buffer is none.	
*/
#define IOCTL_COMSYNC_CLEAR_RTS\
	CTL_CODE(FILE_DEVICE_CTI, 0x81b, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This IOCTL will return the CTS state.

	- Input buffer is none.	    
	- Output buffer is CTS state by UCHAR.		
*/
#define IOCTL_COMSYNC_GET_CTS\
	CTL_CODE(FILE_DEVICE_CTI, 0x81c, METHOD_BUFFERED, FILE_ANY_ACCESS)

/*
	This IOCTL will return the DCD state.

	- Input buffer is none.	    
	- Output buffer is DCD state by UCHAR.	
*/
#define IOCTL_COMSYNC_GET_DCD\
	CTL_CODE(FILE_DEVICE_CTI, 0x81d, METHOD_BUFFERED, FILE_ANY_ACCESS)

#endif